export default {
	/**
	 * Converts an integer (0 to 16581375) into a RGB color string (e.g. #012345)
	 * @param   {Number} colorInt
	 * @returns {String}
	 */
	intToHexString: function(colorInt) {
		var r = Math.floor(colorInt / (255 * 255)).toString(16);
		var g = Math.floor(colorInt / 255).toString(16);
		var b = Math.floor(colorInt % 255).toString(16);
		return (
			"#" +
			(r.length == 1 ? "0" + r : r) +
			(g.length == 1 ? "0" + g : g) +
			(b.length == 1 ? "0" + b : b)
		);
	},
	/**
	 * Converts RGB values (numbers 0 to 255) into a RGB color string (e.g. #012345)
	 * @param   {Number} r
	 * @param   {Number} g
	 * @param   {Number} b
	 * @returns {String}
	 */
	rgbToHexString: function(r, g, b) {
		r = r.toString(16);
		g = g.toString(16);
		b = b.toString(16);
		return (
			"#" +
			(r.length == 1 ? "0" + r : r) +
			(g.length == 1 ? "0" + g : g) +
			(b.length == 1 ? "0" + b : b)
		);
	},
	/**
	 * Calculate the foreground color based on the given hex background color.
	 * See colors.xsl.
	 * @param {String} hexColor - Hex color, including # (e.g. #012345).
	 */
	calculateForegroundColor: function(hexColor) {
		switch (hexColor) {
			case "#d3d3d3": //COLOR_LIGHT_GRAY
			case "#f0e68c": //COLOR_KHAKI
			case "#ffd700": //COLOR_YELLOW
			case "#ffb6c1": //COLOR_PINK
			case "#87cefa": //COLOR_LIGHT_BLUE
			case "#56e756": //COLOR_LIME
				return "#334444";
			case "#79b400": //COLOR_ESKO_GREEN
			case "#f79d00": //COLOR_ORANGE
			case "#cd853f": //COLOR_SAND
			case "#8b4513": //COLOR_BROWN
			case "#800000": //COLOR_DARK_RED
			case "#ff3000": //COLOR_RED
			case "#ff00ff": //COLOR_MAGENTA
			case "#ee82ee": //COLOR_VIOLET
			case "#5a009c": //COLOR_INDIGO
			case "#2e2eff": //COLOR_BLUE
			case "#6699ff": //COLOR_MEDIUM_BLUE
			case "#9acd32": //COLOR_GREEN
			case "#6b8e23": //COLOR_OLIVE
			case "#056e05": //COLOR_DARK_GREEN
			/* falls through */
			default:
				return "#FFFFFF";
		}
	},

	/**
	 * Convert a HEX string to RGB
	 * @param   {String} hex
	 * @returns {Object} {r: 255, g: 255, b: 255}
	 */
	hexToRgb: function(hex) {
		// Expand shorthand form (e.g. "03F") to full form (e.g. "0033FF")
		var shorthandRegex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
		hex = hex.replace(shorthandRegex, function(m, r, g, b) {
			return r + r + g + g + b + b;
		});

		var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
		return result
			? {
					r: parseInt(result[1], 16),
					g: parseInt(result[2], 16),
					b: parseInt(result[3], 16)
			  }
			: null;
	},

	/**
	 *
	 * @param   {String} hexColor
	 * @param   {Number} [threshold=.73] - i.e.  0.75
	 * @returns {String} hexColor (white or black)
	 */
	calculateContrastColor: function(hexColor, threshold) {
		if (typeof threshold === "undefined") {
			threshold = 0.73;
		}

		var rgb = this.hexToRgb(hexColor);
		var arr = [rgb.r, rgb.g, rgb.b];

		for (var i = 0; i < arr.length; i++) {
			arr[i] = arr[i] / 255;
			if (arr[i] <= 0.03928) {
				arr[i] = arr[i] / 12.92;
			} else {
				arr[i] = Math.pow((arr[i] + 0.055) / 1.055, 2.4);
			}
		}

		var luminance = 0.2126 * arr[0] + 0.7152 * arr[1] + 0.0722 * arr[2];
		if (luminance > threshold) {
			return "#000000";
		} else {
			return "#ffffff";
		}
	},

	/**
	 * This is a color set, curated by DEKE
	 *
	 * It contains some Esko Brand colors and nice vivid colors which can be used to hash color strings
	 * See also hashStringToColor
	 */
	DEFAULT_COLOR_SET: [
		// Shades of Grey
		"#293135",
		"#444b53",
		"#668788",
		"#b3b2b3",
		"#dbb583",
		// Color Tool
		"#8a0042",
		"#bf1e6d",
		"#f65a9b",
		"#9f1e26",
		"#d6524e",
		"#ff837a",
		"#ac2500",
		"#e55825",
		"#ff8952",
		"#b67200",
		"#eda135",
		"#ffd266",
		"#bfb900",
		"#f6eb0f",
		"#ffff59",
		"#878e00",
		"#bbbe19",
		"#f1f054",
		"#0a751b",
		"#4aa548",
		"#7dd776",
		"#0088a8",
		"#46b8da",
		"#80eaff",
		"#004f8b",
		"#167bbb",
		"#5daaee",
		// Material
		"#F44336",
		"#E91E63",
		"#9C27B0",
		"#673AB7",
		"#3F51B5",
		"#2196F3",
		"#03A9F4",
		"#00BCD4",
		"#009688",
		"#4CAF50",
		"#8BC34A",
		"#CDDC39",
		"#FFEB3B",
		"#FFC107",
		"#FF9800",
		"#FF5722",
		"#795548",
		"#9E9E9E",
		"#607D8B"
	],

	/**
	 * Convert to 32-bit number
	 * @param   {string}   str - string to be encoded
	 * @returns {Number} hash for str
	 */
	hashString: function(str) {
		var hash = 0;
		var strLen = str.length;

		for (var i = 0; i < strLen; i++) {
			var char = str.charCodeAt(i);
			hash = (hash << 5) - hash + char;
			hash = hash & hash; // Convert to 32bit integer
		}
		return hash;
	},

	/**
	 * Hash a string to a "random" color, the same string will always return the same color
	 * (collission can (and will) occur
	 *
	 * Color set is optional and by default ColorUtils.DEFAULT_COLOR_SET is used
	 *
	 * @param   {String}         str
	 * @param   {Array.<String>} [colorSet=ColorUtils.DEFAULT_COLOR_SET] - set of (hexadecimal encoded) colors (if there are doubles in this set, there is a much higher risk for collisions!)
	 * @returns {String}         the hashed color value
	 */
	hashStringToColor: function(str, colorSet) {
		if (!colorSet) {
			colorSet = this.DEFAULT_COLOR_SET;
		}

		return colorSet[Math.abs(this.hashString(str)) % colorSet.length];
	}
};
